var system = require("../../../system");
var settings = require("../../../../config/settings");
const AppServiceBase = require("../../app.base");
const querystring = require('querystring');
const md5 = require("MD5");
const NodeRSA = require('node-rsa');

const crypto = require("crypto");
class utilsTlBankSve extends AppServiceBase {
  constructor() {
    super();
    this.logCtl = system.getObject("service.common.oplogSve");
    this.apppaliparamDao = system.getObject("db.dbapp.apppaliparamDao");

  }
  //-------------------------------------------钉钉h5支付-----------------开始

  /**
   * 唤起钉钉h5支付信息---只供简单应用
   * @param {*} pobj 
   * @param {*} actionBody {total_fee:总额，单位为分，order_num:订单号，body_desc:标题描述}
   */
  async getH5AliDingPayInfo(pobj, actionBody) {
    var payParam = await this.apppaliparamDao.getItemByUAppId(pobj.appInfo.uapp_id);
    if (!payParam) {
      return { status: -310, msg: "应用没有对应的支付凭证" };
    }
    var parameters = {
      service: "mobile.securitypay.pay",
      partner: payParam.partner,
      _input_charset: "utf-8",
      notify_url: payParam.notify_url,
      // app_id: pobj.appInfo.uapp_id.toString(),//可为空
      out_trade_no: actionBody.order_num,
      subject: actionBody.body_desc,
      payment_type: "1",
      seller_id: payParam.seller_id,
      total_fee: actionBody.total_fee,
      body: actionBody.body_desc,
      sign_type: "RSA"
    };
    // var signStr = await this.getCreateAliSignStr(payParam, parameters);
    var signStr = await this.generateSign(pobj, parameters, payParam.privatekey, "RSA-SHA1");
    parameters.sign = signStr;
    return system.getResultSuccess(parameters);
  }

  async getCreateAliSignStr(payParam, parameters) {
    let userPrivateKey = payParam.privatekey;
    const key = new NodeRSA();
    key.setOptions({ b: 1024, signingScheme: "sha1" });
    key.importKey(userPrivateKey, 'pkcs8-private');//导入密钥并设定格式
    const publicDer = key.exportKey('pkcs8-public');
    let text = Object.keys(parameters).sort().map(key => {
      return `${encodeURIComponent(key)}=${encodeURIComponent(parameters[key])}`
    }).join('&');
    let buffer = Buffer.from(text);
    let signature = key.sign(buffer).toString('base64');
    return signature;
  }


  /**
 * 生成签名
 * 支付宝官方文档：https://doc.open.alipay.com/docs/doc.htm?docType=1&articleId=106118
 * 筛选排序 -> 拼接 -> 签名
 *
 * @param {Object} dataObj 待签名的数据对象
 * @param {String} privateKey 私钥
 * @returns {String}
 */
  async generateSign(pobj, dataObj, privateKey, algorith) {
    const stringToBeSign = Object.keys(dataObj).sort().reduce((acc, k) => {
      if (k !== 'sign' && dataObj[k] !== undefined) {
        const v = `${dataObj[k]}`;
        if (v.length !== 0) {
          if (acc.length === 0) {
            acc += `${k}=${v}`;
          } else {
            acc += `&${k}=${v}`;
          }
        }
      }
      return acc;
    }, '');

    if (stringToBeSign.length === 0) {
      throw new Error('The data to be signed is empty.');
    } else {
      const cryptoSign = crypto.createSign(algorith);
      cryptoSign.update(stringToBeSign, 'utf8');
      var signStr = cryptoSign.sign(privateKey, 'base64');

      this.logCtl.info({
        appid: pobj.appInfo ? pobj.appInfo.uapp_id : "",
        appkey: pobj.appInfo ? pobj.appInfo.uapp_key : "",
        op: "center-app/app/base/service/impl/utilsSve/utilsAlibankSve.js/generateSign",
        content: "参数：" + stringToBeSign + "返回结果signStr：" + signStr,
        optitle: "获取钉钉支付签名信息返回",
      });
      return signStr;
    }
  }

  //-------------------------------------------钉钉h5支付-----------------结束

}
module.exports = utilsTlBankSve;
